@@ -23,6 +23,7 @@
 
 #include <thunar/thunar-clipboard-manager.h>
 #include <thunar/thunar-gobject-extensions.h>
+#include <thunar/thunar-gdk-extensions.h>
 #include <thunar/thunar-icon-factory.h>
 #include <thunar/thunar-icon-renderer.h>
 #include <thunar/thunar-private.h>
@@ -333,6 +334,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   GList                  *lp;
   gint                    max_emblems;
   gint                    position;
+  cairo_t                *cr;
+  gdouble                 alpha;
+  gint                    emblem_size;
 
   if (G_UNLIKELY (icon_renderer->file == NULL))
     return;
@@ -366,7 +370,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   if (G_UNLIKELY (icon_area.width > cell_area->width || icon_area.height > cell_area->height))
     {
       /* scale down to fit */
-      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, cell_area->width, cell_area->height);
+      temp = exo_gdk_pixbuf_scale_down (icon, TRUE, MAX (1, cell_area->width), MAX (1, cell_area->height));
       g_object_unref (G_OBJECT (icon));
       icon = temp;
 
@@ -378,6 +382,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
   icon_area.x = cell_area->x + (cell_area->width - icon_area.width) / 2;
   icon_area.y = cell_area->y + (cell_area->height - icon_area.height) / 2;
 
+  /* create the context */
+  cr = gdk_cairo_create (window);
+
   /* check whether the icon is affected by the expose event */
   if (gdk_rectangle_intersect (expose_area, &icon_area, &draw_area))
     {
@@ -386,16 +393,16 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
       if (thunar_clipboard_manager_has_cutted_file (clipboard, icon_renderer->file))
         {
           /* 50% translucent for cutted files */
-          temp = exo_gdk_pixbuf_lucent (icon, 50);
-          g_object_unref (G_OBJECT (icon));
-          icon = temp;
+          alpha = 0.50;
         }
       else if (thunar_file_is_hidden (icon_renderer->file))
         {
           /* 75% translucent for hidden files */
-          temp = exo_gdk_pixbuf_lucent (icon, 75);
-          g_object_unref (G_OBJECT (icon));
-          icon = temp;
+          alpha = 0.75;
+        }
+      else
+        {
+          alpha = 1.00;
         }
       g_object_unref (G_OBJECT (clipboard));
 
@@ -404,7 +411,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         {
           if ((flags & GTK_CELL_RENDERER_SELECTED) != 0)
             {
-              state = GTK_WIDGET_HAS_FOCUS (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
+              state = gtk_widget_has_focus (widget) ? GTK_STATE_SELECTED : GTK_STATE_ACTIVE;
               temp = exo_gdk_pixbuf_colorize (icon, &widget->style->base[state]);
               g_object_unref (G_OBJECT (icon));
               icon = temp;
@@ -419,7 +426,7 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
 
       /* check if we should render an insensitive icon */
-      if (G_UNLIKELY (GTK_WIDGET_STATE (widget) == GTK_STATE_INSENSITIVE || !renderer->sensitive))
+      if (G_UNLIKELY (gtk_widget_get_state (widget) == GTK_STATE_INSENSITIVE || !renderer->sensitive))
         {
           /* allocate an icon source */
           icon_source = gtk_icon_source_new ();
@@ -438,10 +445,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
 
       /* render the invalid parts of the icon */
-      gdk_draw_pixbuf (window, widget->style->black_gc, icon,
-                       draw_area.x - icon_area.x, draw_area.y - icon_area.y,
-                       draw_area.x, draw_area.y, draw_area.width, draw_area.height,
-                       GDK_RGB_DITHER_NORMAL, 0, 0);
+      thunar_gdk_cairo_set_source_pixbuf (cr, icon, icon_area.x, icon_area.y);
+      gdk_cairo_rectangle (cr, &draw_area);
+      cairo_paint_with_alpha (cr, alpha);
     }
 
   /* release the file's icon */
@@ -460,8 +466,11 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
           /* render the emblems */
           for (lp = emblems, position = 0; lp != NULL && position < max_emblems; lp = lp->next)
             {
+              /* calculate the emblem size */
+              emblem_size = MIN ((2 * icon_renderer->size) / 3, 32);
+
               /* check if we have the emblem in the icon theme */
-              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, icon_renderer->size, NULL, FALSE);
+              emblem = thunar_icon_factory_load_icon (icon_factory, lp->data, emblem_size, FALSE);
               if (G_UNLIKELY (emblem == NULL))
                 continue;
 
@@ -470,10 +479,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               emblem_area.height = gdk_pixbuf_get_height (emblem);
 
               /* shrink insane emblems */
-              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > (gint) MIN ((2 * icon_renderer->size) / 3, 36)))
+              if (G_UNLIKELY (MAX (emblem_area.width, emblem_area.height) > emblem_size))
                 {
                   /* scale down the emblem */
-                  temp = exo_gdk_pixbuf_scale_ratio (emblem, MIN ((2 * icon_renderer->size) / 3, 36));
+                  temp = exo_gdk_pixbuf_scale_ratio (emblem, emblem_size);
                   g_object_unref (G_OBJECT (emblem));
                   emblem = temp;
 
@@ -520,10 +529,10 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
               /* render the emblem */
               if (gdk_rectangle_intersect (expose_area, &emblem_area, &draw_area))
                 {
-                  gdk_draw_pixbuf (window, widget->style->black_gc, emblem,
-                                   draw_area.x - emblem_area.x, draw_area.y - emblem_area.y,
-                                   draw_area.x, draw_area.y, draw_area.width, draw_area.height,
-                                   GDK_RGB_DITHER_NORMAL, 0, 0);
+                  /* render the invalid parts of the icon */
+                  thunar_gdk_cairo_set_source_pixbuf (cr, emblem, emblem_area.x, emblem_area.y);
+                  gdk_cairo_rectangle (cr, &draw_area);
+                  cairo_paint (cr);
                 }
 
               /* release the emblem */
@@ -538,6 +547,9 @@ thunar_icon_renderer_render (GtkCellRenderer     *renderer,
         }
     }
 
+  /* destroy the context */
+  cairo_destroy (cr);
+
   /* release our reference on the icon factory */
   g_object_unref (G_OBJECT (icon_factory));
 }
