@@ -114,6 +114,7 @@ static gboolean           thunar_shortcuts_model_drag_data_get      (GtkTreeDrag
                                                                      GtkSelectionData          *selection_data);
 static gboolean           thunar_shortcuts_model_drag_data_delete   (GtkTreeDragSource         *source,
                                                                      GtkTreePath               *path);
+static void               thunar_shortcuts_model_header_visibility  (ThunarShortcutsModel      *model);
 static void               thunar_shortcuts_model_shortcut_places    (ThunarShortcutsModel      *model);
 static void               thunar_shortcuts_model_shortcut_network   (ThunarShortcutsModel      *model);
 static gboolean           thunar_shortcuts_model_devices_load       (gpointer                   data);
@@ -393,6 +394,9 @@ thunar_shortcuts_model_set_property (GObject      *object,
               gtk_tree_path_free (path);
             }
         }
+
+      /* update header visibility */
+      thunar_shortcuts_model_header_visibility (model);
       break;
 
     default:
@@ -425,10 +429,10 @@ thunar_shortcuts_model_get_column_type (GtkTreeModel *tree_model,
 {
   switch (idx)
     {
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER:
       return G_TYPE_BOOLEAN;
 
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM:
       return G_TYPE_BOOLEAN;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE:
@@ -541,22 +545,19 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
 
   switch (column)
     {
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value, (shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) != 0);
       break;
 
-    case THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM:
+    case THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value, (shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) == 0);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE:
       g_value_init (value, G_TYPE_BOOLEAN);
-      if (shortcut->device == NULL)
-        g_value_set_boolean (value, !shortcut->hidden);
-      else
-        g_value_set_boolean (value, !thunar_device_get_hidden (shortcut->device));
+      g_value_set_boolean (value, !shortcut->hidden);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_NAME:
@@ -650,8 +651,7 @@ thunar_shortcuts_model_get_value (GtkTreeModel *tree_model,
     case THUNAR_SHORTCUTS_MODEL_COLUMN_MUTABLE:
       g_value_init (value, G_TYPE_BOOLEAN);
       g_value_set_boolean (value,
-                           shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                           || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+                           shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
       break;
 
     case THUNAR_SHORTCUTS_MODEL_COLUMN_CAN_EJECT:
@@ -817,6 +817,66 @@ thunar_shortcuts_model_drag_data_delete (GtkTreeDragSource *source,
 
 
 
+static void
+thunar_shortcuts_model_header_visibility (ThunarShortcutsModel *model)
+{
+  GList          *lp;
+  ThunarShortcut *shortcut;
+  ThunarShortcut *header;
+  guint           n_children = 0;
+  GList          *lp_header = NULL;
+  gint            idx_header = 0;
+  gint            i;
+  GtkTreePath    *path;
+  GtkTreeIter     iter;
+
+  _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
+
+  for (lp = model->shortcuts, i = 0; lp != NULL; lp = lp->next, i++)
+    {
+      shortcut = lp->data;
+
+      if ((shortcut->group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
+        {
+          if (lp_header != NULL)
+            {
+              update_header:
+
+              header = lp_header->data;
+              if (header->hidden != (n_children == 0))
+                {
+                  /* set new visibility */
+                  header->hidden = (n_children == 0);
+
+                  /* notify view */
+                  path = gtk_tree_path_new_from_indices (idx_header, -1);
+                  GTK_TREE_ITER_INIT (iter, model->stamp, lp_header);
+                  gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
+                  gtk_tree_path_free (path);
+                }
+
+              if (lp == NULL)
+                return;
+            }
+
+          /* reset for new */
+          idx_header = i;
+          lp_header = lp;
+          n_children = 0;
+        }
+      else if (!shortcut->hidden)
+        {
+          n_children++;
+        }
+    }
+
+  _thunar_assert (lp == NULL);
+  if (lp_header != NULL)
+    goto update_header;
+}
+
+
+
 static gboolean
 thunar_shortcuts_model_devices_load (gpointer data)
 {
@@ -849,7 +909,7 @@ thunar_shortcuts_model_devices_load (gpointer data)
   devices = thunar_device_monitor_get_devices (model->device_monitor);
   for (lp = devices; lp != NULL; lp = lp->next)
     {
-      thunar_shortcuts_model_device_added (model->device_monitor, lp->data, model);
+      thunar_shortcuts_model_device_added (NULL, lp->data, model);
       g_object_unref (G_OBJECT (lp->data));
     }
   g_list_free (devices);
@@ -861,6 +921,8 @@ thunar_shortcuts_model_devices_load (gpointer data)
   g_signal_connect (model->device_monitor, "device-removed", G_CALLBACK (thunar_shortcuts_model_device_removed), model);
   g_signal_connect (model->device_monitor, "device-changed", G_CALLBACK (thunar_shortcuts_model_device_changed), model);
 
+  thunar_shortcuts_model_header_visibility (model);
+
   model->devices_monitor_idle_id = 0;
 
   return FALSE;
@@ -1112,8 +1174,7 @@ thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
       gtk_tree_path_free (path);
 
       /* check if we need to save */
-      needs_save = shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                   || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      needs_save = (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
       /* actually free the shortcut */
       thunar_shortcut_free (shortcut, model);
@@ -1126,6 +1187,22 @@ thunar_shortcuts_model_remove_shortcut (ThunarShortcutsModel *model,
 
 
 
+static gboolean
+thunar_shortcuts_model_local_file (GFile *gfile)
+{
+  _thunar_return_val_if_fail (G_IS_FILE (gfile), FALSE);
+
+  /* schemes we'd like to have as ThunarFiles in the model */
+  if (g_file_has_uri_scheme (gfile, "file")
+      || g_file_has_uri_scheme (gfile, "computer")
+      || g_file_has_uri_scheme (gfile, "recent"))
+    return TRUE;
+
+  return FALSE;
+}
+
+
+
 static void
 thunar_shortcuts_model_load_line (GFile       *file_path,
                                   const gchar *name,
@@ -1141,7 +1218,7 @@ thunar_shortcuts_model_load_line (GFile       *file_path,
   _thunar_return_if_fail (name == NULL || g_utf8_validate (name, -1, NULL));
 
   /* handle local and remove files differently */
-  if (g_file_has_uri_scheme (file_path, "file"))
+  if (thunar_shortcuts_model_local_file (file_path))
     {
       /* try to open the file corresponding to the uri */
       file = thunar_file_get (file_path, NULL);
@@ -1171,10 +1248,11 @@ thunar_shortcuts_model_load_line (GFile       *file_path,
     {
       /* create the shortcut entry */
       shortcut = g_slice_new0 (ThunarShortcut);
-      shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
       shortcut->gicon = g_themed_icon_new ("folder-remote");
       shortcut->location = g_object_ref (file_path);
       shortcut->sort_id = row_num;
+      shortcut->hidden = thunar_shortcuts_model_get_hidden (model, shortcut);
       shortcut->name = g_strdup (name);
 
       /* append the shortcut to the list */
@@ -1198,6 +1276,9 @@ thunar_shortcuts_model_load (gpointer data)
                               thunar_shortcuts_model_load_line,
                               model);
 
+  /* update the visibility */
+  thunar_shortcuts_model_header_visibility (model);
+
   GDK_THREADS_LEAVE ();
 
   model->bookmarks_idle_id = 0;
@@ -1297,8 +1378,7 @@ thunar_shortcuts_model_save (ThunarShortcutsModel *model)
   for (lp = model->shortcuts; lp != NULL; lp = lp->next)
     {
       shortcut = THUNAR_SHORTCUT (lp->data);
-      if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-          || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+      if (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
         {
           if (shortcut->file != NULL)
             uri = thunar_file_dup_uri (shortcut->file);
@@ -1406,8 +1486,8 @@ thunar_shortcuts_model_device_added (ThunarDeviceMonitor  *device_monitor,
 {
   ThunarShortcut *shortcut;
 
-  _thunar_return_if_fail (THUNAR_DEVICE_MONITOR (device_monitor));
-  _thunar_return_if_fail (model->device_monitor == device_monitor);
+  _thunar_return_if_fail (device_monitor == NULL || THUNAR_DEVICE_MONITOR (device_monitor));
+  _thunar_return_if_fail (device_monitor == NULL || model->device_monitor == device_monitor);
   _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_MODEL (model));
 
@@ -1433,6 +1513,11 @@ thunar_shortcuts_model_device_added (ThunarDeviceMonitor  *device_monitor,
 
   /* insert in the model */
   thunar_shortcuts_model_add_shortcut (model, shortcut);
+
+  /* header visibility if call is from monitor */
+  if (device_monitor != NULL
+      && !shortcut->hidden)
+    thunar_shortcuts_model_header_visibility (model);
 }
 
 
@@ -1475,6 +1560,7 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
   gint            idx;
   GtkTreePath    *path;
   ThunarShortcut *shortcut;
+  gboolean        update_header = FALSE;
 
   _thunar_return_if_fail (THUNAR_DEVICE_MONITOR (device_monitor));
   _thunar_return_if_fail (model->device_monitor == device_monitor);
@@ -1498,6 +1584,13 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
       g_free (shortcut->tooltip);
       shortcut->tooltip = NULL;
 
+      /* hidden state */
+      if (shortcut->hidden != thunar_device_get_hidden (device))
+        {
+          shortcut->hidden = thunar_device_get_hidden (device);
+          update_header = TRUE;
+        }
+
       /* generate an iterator for the path */
       GTK_TREE_ITER_INIT (iter, model->stamp, lp);
 
@@ -1506,6 +1599,10 @@ thunar_shortcuts_model_device_changed (ThunarDeviceMonitor  *device_monitor,
       gtk_tree_model_row_changed (GTK_TREE_MODEL (model), path, &iter);
       gtk_tree_path_free (path);
     }
+
+  /* header visibility */
+  if (update_header)
+    thunar_shortcuts_model_header_visibility (model);
 }
 
 
@@ -1649,8 +1746,7 @@ thunar_shortcuts_model_has_bookmark (ThunarShortcutsModel *model,
       shortcut = lp->data;
 
       /* only check bookmarks */
-      if (shortcut->group != THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-          && shortcut->group != THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+      if (shortcut->group != THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
         continue;
 
       if (shortcut->file != NULL
@@ -1806,22 +1902,16 @@ thunar_shortcuts_model_add (ThunarShortcutsModel *model,
 
   /* create the new shortcut that will be inserted */
   shortcut = g_slice_new0 (ThunarShortcut);
+  shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
 
-  if (THUNAR_IS_FILE (file) && thunar_file_is_local (file))
-    shortcut->file = g_object_ref (G_OBJECT (file));
-  else
-    shortcut->location = g_object_ref (G_OBJECT (location));
-
-  if (g_file_has_uri_scheme (location, "file"))
+  if (thunar_shortcuts_model_local_file (location))
     {
-      shortcut->group = THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS;
+      shortcut->file = thunar_file_get (location, NULL);
     }
   else
     {
-      shortcut->group = THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS;
+      shortcut->location = g_object_ref (G_OBJECT (location));
       shortcut->gicon = g_themed_icon_new ("folder-remote");
-
-      dst_path = NULL;
     }
 
   /* add the shortcut to the list at the given position */
@@ -1952,8 +2042,7 @@ thunar_shortcuts_model_remove (ThunarShortcutsModel *model,
   shortcut = g_list_nth_data (model->shortcuts, gtk_tree_path_get_indices (path)[0]);
 
   /* verify that the shortcut is removable */
-  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                  || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
   /* remove the shortcut (using the file destroy handler) */
   thunar_shortcuts_model_remove_shortcut (model, shortcut);
@@ -1991,8 +2080,7 @@ thunar_shortcuts_model_rename (ThunarShortcutsModel *model,
   shortcut = THUNAR_SHORTCUT (((GList *) iter->user_data)->data);
 
   /* verify the shortcut */
-  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-                  || shortcut->group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS);
+  _thunar_assert (shortcut->group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS);
 
   /* perform the rename */
   if (G_UNLIKELY (shortcut->name != NULL))
@@ -2125,4 +2213,7 @@ thunar_shortcuts_model_set_hidden (ThunarShortcutsModel *model,
   /* store new list */
   g_object_set (G_OBJECT (model->preferences), "hidden-bookmarks", bookmarks, NULL);
   g_strfreev (bookmarks);
+
+  /* header visibility */
+  thunar_shortcuts_model_header_visibility (model);
 }
