@@ -113,6 +113,9 @@ static gboolean       thunar_shortcuts_view_selection_func               (GtkTre
                                                                           GtkTreePath              *path,
                                                                           gboolean                  path_currently_selected,
                                                                           gpointer                  user_data);
+static void           thunar_shortcuts_view_context_menu_visibility      (ThunarShortcutsView      *view,
+                                                                          GdkEventButton           *event,
+                                                                          GtkTreeModel             *model);
 static void           thunar_shortcuts_view_context_menu                 (ThunarShortcutsView      *view,
                                                                           GdkEventButton           *event,
                                                                           GtkTreeModel             *model,
@@ -304,7 +307,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "text", THUNAR_SHORTCUTS_MODEL_COLUMN_NAME,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER,
                                        NULL);
 
   /* separator for indent */
@@ -312,7 +315,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   g_object_set (G_OBJECT (renderer), "xpad", 6, NULL);
   gtk_tree_view_column_pack_start (column, renderer, FALSE);
   gtk_tree_view_column_set_attributes (column, renderer,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* allocate the special icon renderer */
@@ -322,7 +325,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
                                        "gicon", THUNAR_SHORTCUTS_MODEL_COLUMN_GICON,
                                        "file", THUNAR_SHORTCUTS_MODEL_COLUMN_FILE,
                                        "device", THUNAR_SHORTCUTS_MODEL_COLUMN_DEVICE,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* sync the "emblems" property of the icon renderer with the "shortcuts-icon-emblems" preference
@@ -339,7 +342,7 @@ thunar_shortcuts_view_init (ThunarShortcutsView *view)
   gtk_tree_view_column_pack_start (column, renderer, TRUE);
   gtk_tree_view_column_set_attributes (column, renderer,
                                        "text", THUNAR_SHORTCUTS_MODEL_COLUMN_NAME,
-                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_ITEM,
+                                       "visible", THUNAR_SHORTCUTS_MODEL_COLUMN_IS_ITEM,
                                        NULL);
 
   /* spinner to indicate (un)mount/eject delay */
@@ -421,6 +424,8 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
   /* let the widget process the event first (handles focussing and scrolling) */
   result = (*GTK_WIDGET_CLASS (thunar_shortcuts_view_parent_class)->button_press_event) (widget, event);
 
+  model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
+
   /* resolve the path at the cursor position */
   if (gtk_tree_view_get_path_at_pos (GTK_TREE_VIEW (widget), event->x, event->y, &path, NULL, NULL, NULL))
     {
@@ -428,7 +433,6 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
       if (G_UNLIKELY (event->button == 3 && event->type == GDK_BUTTON_PRESS))
         {
           /* determine the iterator for the path */
-          model = gtk_tree_view_get_model (GTK_TREE_VIEW (view));
           if (gtk_tree_model_get_iter (model, &iter, path))
             {
               /* popup the context menu */
@@ -464,6 +468,11 @@ thunar_shortcuts_view_button_press_event (GtkWidget      *widget,
       /* release the path */
       gtk_tree_path_free (path);
     }
+  else if (event->button == 3)
+    {
+      thunar_shortcuts_view_context_menu_visibility (view, event, model);
+      result = TRUE;
+    }
 
   return result;
 }
@@ -912,15 +921,15 @@ thunar_shortcuts_view_selection_func (GtkTreeSelection *selection,
 
   /* don't allow selecting headers */
   gtk_tree_model_get_iter (model, &iter, path);
-  gtk_tree_model_get (model, &iter, THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER, &is_header, -1);
+  gtk_tree_model_get (model, &iter, THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER, &is_header, -1);
   return !is_header;
 }
 
 
 
 static void
-thunar_shortcuts_view_context_menu_header_toggled (GtkCheckMenuItem *item,
-                                                   GtkTreeModel     *model)
+thunar_shortcuts_view_context_menu_visibility_toggled (GtkCheckMenuItem *item,
+                                                       GtkTreeModel     *model)
 {
   gboolean             hidden;
   GtkTreePath         *path;
@@ -942,10 +951,9 @@ thunar_shortcuts_view_context_menu_header_toggled (GtkCheckMenuItem *item,
 
 
 static void
-thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
-                                           GdkEventButton      *event,
-                                           GtkTreeModel        *model,
-                                           GtkTreeIter         *header_iter)
+thunar_shortcuts_view_context_menu_visibility (ThunarShortcutsView *view,
+                                               GdkEventButton      *event,
+                                               GtkTreeModel        *model)
 {
   GtkTreeIter          iter;
   guint                mask = 0;
@@ -957,17 +965,18 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
   gboolean             visible;
   GtkTreePath         *path;
   GtkTreeModel        *child_model;
+  GtkWidget           *submenu;
 
   /* unselect the items */
   selection = gtk_tree_view_get_selection (GTK_TREE_VIEW (view));
   gtk_tree_selection_unselect_all (GTK_TREE_SELECTION (selection));
 
   /* prepare the popup menu */
-  menu = gtk_menu_new ();
+  menu = submenu = gtk_menu_new ();
 
   /* process all items below the header */
   child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
-  gtk_tree_model_filter_convert_iter_to_child_iter (GTK_TREE_MODEL_FILTER (model), &iter, header_iter);
+  gtk_tree_model_get_iter_first (GTK_TREE_MODEL (child_model), &iter);
   path = gtk_tree_model_get_path (child_model, &iter);
   do
     {
@@ -978,7 +987,7 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
                           THUNAR_SHORTCUTS_MODEL_COLUMN_VISIBLE, &visible,
                           -1);
 
-      if (mask == 0 && (group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
+      if ((group & THUNAR_SHORTCUT_GROUP_HEADER) != 0)
         {
           /* get the mask of the group */
           if ((group & THUNAR_SHORTCUT_GROUP_DEVICES) != 0)
@@ -993,23 +1002,21 @@ thunar_shortcuts_view_context_menu_header (ThunarShortcutsView *view,
           /* create menu items */
           mi = gtk_menu_item_new_with_label (label);
           gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-          gtk_widget_set_sensitive (mi, FALSE);
           gtk_widget_show (mi);
 
-          mi = gtk_separator_menu_item_new ();
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
-          gtk_widget_show (mi);
+          submenu = gtk_menu_new ();
+          gtk_menu_item_set_submenu (GTK_MENU_ITEM (mi), submenu);
         }
       else if ((group & mask) != 0)
         {
           mi = gtk_check_menu_item_new_with_label (label);
-          gtk_menu_shell_append (GTK_MENU_SHELL (menu), mi);
+          gtk_menu_shell_append (GTK_MENU_SHELL (submenu), mi);
           gtk_check_menu_item_set_active (GTK_CHECK_MENU_ITEM (mi), visible);
           g_object_set_data_full (G_OBJECT (mi), I_("thunar-shortcuts-row"),
                                   gtk_tree_row_reference_new (child_model, path),
                                   (GDestroyNotify) gtk_tree_row_reference_free);
           g_signal_connect (G_OBJECT (mi), "toggled",
-            G_CALLBACK (thunar_shortcuts_view_context_menu_header_toggled), child_model);
+            G_CALLBACK (thunar_shortcuts_view_context_menu_visibility_toggled), child_model);
           gtk_widget_show (mi);
         }
 
@@ -1050,10 +1057,10 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
   GtkTreeModel        *shortcuts_model;
 
   /* check if this is an item menu or a header menu */
-  gtk_tree_model_get (model, iter, THUNAR_SHORTCUTS_MODEL_COLUMN_HEADER, &is_header, -1);
+  gtk_tree_model_get (model, iter, THUNAR_SHORTCUTS_MODEL_COLUMN_IS_HEADER, &is_header, -1);
   if (is_header)
     {
-      thunar_shortcuts_view_context_menu_header (view, event, model, iter);
+      thunar_shortcuts_view_context_menu_visibility (view, event, model);
       return;
     }
 
@@ -1226,8 +1233,7 @@ thunar_shortcuts_view_context_menu (ThunarShortcutsView *view,
     }
 
   /* append the remove menu item */
-  if (group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS
-      || group == THUNAR_SHORTCUT_GROUP_NETWORK_BOOKMARKS)
+  if (group == THUNAR_SHORTCUT_GROUP_PLACES_BOOKMARKS)
     {
       /* append a menu separator */
       item = gtk_separator_menu_item_new ();
@@ -1856,7 +1862,6 @@ thunar_shortcuts_view_eject (ThunarShortcutsView *view)
   GtkTreeIter       iter;
   ThunarDevice     *device;
   GMountOperation  *mount_operation;
-  GtkWidget        *window;
   GtkTreeModel     *child_model;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
@@ -1870,8 +1875,7 @@ thunar_shortcuts_view_eject (ThunarShortcutsView *view)
       _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
 
       /* prepare a mount operation */
-      window = gtk_widget_get_toplevel (GTK_WIDGET (view));
-      mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
+      mount_operation = thunar_gtk_mount_operation_new (GTK_WIDGET (view));
 
       /* start the spinner */
       child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
@@ -2000,7 +2004,6 @@ thunar_shortcuts_view_unmount (ThunarShortcutsView *view)
   GtkTreeIter       iter;
   ThunarDevice     *device;
   GMountOperation  *mount_operation;
-  GtkWidget        *window;
   GtkTreeModel     *child_model;
 
   _thunar_return_if_fail (THUNAR_IS_SHORTCUTS_VIEW (view));
@@ -2014,8 +2017,7 @@ thunar_shortcuts_view_unmount (ThunarShortcutsView *view)
       _thunar_return_if_fail (THUNAR_IS_DEVICE (device));
 
       /* prepare a mount operation */
-      window = gtk_widget_get_toplevel (GTK_WIDGET (view));
-      mount_operation = gtk_mount_operation_new (GTK_WINDOW (window));
+      mount_operation = thunar_gtk_mount_operation_new (GTK_WIDGET (view));
 
       /* start the spinner */
       child_model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (model));
